/*
 * taskview-generic-proxy.c
 *
 * TaskviewGenericProxy
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include "taskview-generic-proxy.h"
#include "taskview.h"

#define TASKVIEW_GENERIC_PROXY_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), TASKVIEW_TYPE_GENERIC_PROXY, TaskviewGenericProxyPrivate))

G_DEFINE_TYPE (TaskviewGenericProxy, taskview_generic_proxy, G_TYPE_OBJECT);

struct _TaskviewGenericProxyPrivate
{
  GDBusConnection *connection;
  GDBusProxy *proxy;
  gchar *dbus_name;
  gchar *object_path;
};

/* ----------------------------------------------------------------------------------------------------
 * GObject Properties
 * ---------------------------------------------------------------------------------------------------- */

enum /* properties */
{
  PROP_0,
  PROP_DBUS_NAME,
  PROP_DBUS_OBJECT_PATH,
  PROP_DBUS_CONNECTION,

  /* Properties of the remote object */

	PROP_ICON,
  PROP_DESCRIPTION,
  PROP_DETAILS,
  PROP_STATE,
  PROP_PROGRESS,
  PROP_REMAINING_TIME,
  PROP_CREATION_TIME,
  PROP_FINISHED_TIME,
  
	PROP_VIEWABLE,
  PROP_PAUSABLE,
  PROP_CANCELLABLE
};

static void
taskview_generic_proxy_set_property (GObject      *object,
                                     guint         property_id,
                                     const GValue *value,
                                     GParamSpec   *pspec)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (object);

  switch (property_id)
    {
    case PROP_DBUS_NAME:
      /* Nothing to free, as it is G_PARAM_CONSTRUCT_ONLY */
      self->priv->dbus_name = g_value_dup_string (value);
      break;

    case PROP_DBUS_OBJECT_PATH:
      /* Nothing to free, as it is G_PARAM_CONSTRUCT_ONLY */
      self->priv->object_path = g_value_dup_string (value);
      break;

    case PROP_DBUS_CONNECTION:
      /* Nothing to free, as it is G_PARAM_CONSTRUCT_ONLY */
      self->priv->connection = G_DBUS_CONNECTION (g_value_get_pointer (value));
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }
}

static void
taskview_generic_proxy_get_property (GObject    *object,
                                     guint       property_id,
                                     GValue     *value,
                                     GParamSpec *pspec)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (object);
  GVariant *result= NULL;
  const gchar* type = NULL;
  const gchar* name = NULL;

  switch (property_id)
    {
    case PROP_DBUS_NAME:
      g_value_set_string (value, self->priv->dbus_name);
      break;

    case PROP_DBUS_OBJECT_PATH:
      g_value_set_string (value, self->priv->object_path);
      break;

    case PROP_DBUS_CONNECTION:
      g_value_set_pointer (value, g_object_ref (self->priv->connection));
      break;
				
	 /* Properties of the remote object */
   case PROP_ICON:
      name = "Icon";
      break;

    case PROP_DESCRIPTION:
      name = "Description";
      break;

    case PROP_DETAILS:
      name = "Details";
      break;

    case PROP_STATE:
      name = "State";
      break;

		case PROP_PROGRESS:
      name = "Progress";
      break;

    case PROP_REMAINING_TIME:
      name = "RemainingTime";
      break;

    case PROP_CREATION_TIME:
      name = "CreationTime";
      break;

    case PROP_FINISHED_TIME:
      name = "FinishedTime";
      break;
        
    case PROP_VIEWABLE:
      name = "Viewable";
      break;

    case PROP_PAUSABLE:
      name = "Pausable";
      break;

    case PROP_CANCELLABLE:
      name = "Cancellable";
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }

	if (self->priv->proxy != NULL && name != NULL)
    {
      result = g_dbus_proxy_get_cached_property (self->priv->proxy, name);
    }

  if (result != NULL)
    {
      type = g_variant_get_type_string (result);

      if (g_strcmp0 (type, "b") == 0)
        g_value_set_boolean (value, g_variant_get_boolean (result));
      else if (g_strcmp0 (type, "i") == 0)
        g_value_set_int (value, g_variant_get_int32 (result));
      else if (g_strcmp0 (type, "x") == 0)
        g_value_set_int64 (value, g_variant_get_int64 (result));
      else if (g_strcmp0 (type, "s") == 0)
        g_value_set_string (value, g_variant_get_string (result, NULL));
      else
        g_warn_if_reached ();

      g_variant_unref (result);
    }
	// HACK to prevent segfaults!
	else if (property_id == PROP_DESCRIPTION || property_id == PROP_ICON || property_id == PROP_DETAILS)
		{
			g_value_set_string (value, "");
		}
}

/* ----------------------------------------------------------------------------------------------------
 * Signal Handler
 * ---------------------------------------------------------------------------------------------------- */

/* Translate the g-properties-changed signal to the notify::property */

static void
on_properties_changed (GDBusProxy          *proxy,
                       GVariant            *changed_properties,
                       const gchar* const  *invalidated_properties,
                       gpointer             user_data)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (user_data);

  if (g_variant_n_children (changed_properties) > 0)
    {
      GVariantIter iter;
      GVariant *value;
      gchar *key;

      g_variant_iter_init (&iter, changed_properties);

      while (g_variant_iter_loop (&iter, "{&sv}", &key, &value))
        {
          if (g_strcmp0 (key, "Icon") == 0)
            g_object_notify (G_OBJECT (self), "icon");
          else if (g_strcmp0 (key, "Description") == 0)
            g_object_notify (G_OBJECT (self), "description");
          else if (g_strcmp0 (key, "Details") == 0)
            g_object_notify (G_OBJECT (self), "details");
          else if (g_strcmp0 (key, "State") == 0)
            g_object_notify (G_OBJECT (self), "state");
          else if (g_strcmp0 (key, "Progress") == 0)
            g_object_notify (G_OBJECT (self), "progress");
          else if (g_strcmp0 (key, "RemainingTime") == 0)
            g_object_notify (G_OBJECT (self), "remaining-time");
          else if (g_strcmp0 (key, "CreationTime") == 0)
            g_object_notify (G_OBJECT (self), "creation-time");
          else if (g_strcmp0 (key, "FinishedTime") == 0)
            g_object_notify (G_OBJECT (self), "finished-time");
          else if (g_strcmp0 (key, "Viewable") == 0)
            g_object_notify (G_OBJECT (self), "viewable");
          else if (g_strcmp0 (key, "Pausable") == 0)
            g_object_notify (G_OBJECT (self), "pausable");
          else if (g_strcmp0 (key, "Cancellable") == 0)
            g_object_notify (G_OBJECT (self), "cancellable");
          else
            g_debug ("Unknown Property changed: %s", key);
        }
    }
}

static void
on_proxy_appeared (GObject *source_object, GAsyncResult *res, gpointer user_data)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (user_data);
  GDBusProxy *proxy;
  GError *error = NULL;

  proxy =  g_dbus_proxy_new_for_bus_finish (res, &error);

  if (proxy == NULL)
    {
      g_printerr ("Error creating proxy: %s\n", error->message);
      g_error_free (error);
      return;
    }

  g_signal_connect (proxy,
                    "g-properties-changed",
                    G_CALLBACK (on_properties_changed),
                    self);

  self->priv->proxy = proxy;

	g_object_notify (G_OBJECT (self), "icon");
  g_object_notify (G_OBJECT (self), "description");
  g_object_notify (G_OBJECT (self), "details");
  g_object_notify (G_OBJECT (self), "state");
  g_object_notify (G_OBJECT (self), "progress");
  g_object_notify (G_OBJECT (self), "remaining-time");
  g_object_notify (G_OBJECT (self), "creation-time");
  g_object_notify (G_OBJECT (self), "finished-time");
  g_object_notify (G_OBJECT (self), "viewable");
  g_object_notify (G_OBJECT (self), "pausable");
  g_object_notify (G_OBJECT (self), "cancellable");
}

/* ----------------------------------------------------------------------------------------------------
 * GObject
 * ---------------------------------------------------------------------------------------------------- */

static void
taskview_generic_proxy_constructed (GObject *gobject)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (gobject);

  g_assert (g_dbus_is_name (self->priv->dbus_name));
  g_assert (G_IS_DBUS_CONNECTION (self->priv->connection));

  g_dbus_proxy_new (self->priv->connection,
                    G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START,
                    NULL, /* FIXME: Add org.gnome.TaskView.? */
                    self->priv->dbus_name,
                    self->priv->object_path,
                    TASKVIEW_GENERIC_INTERFACE_NAME,
                    NULL,
                    on_proxy_appeared,
                    self);

  if (G_OBJECT_CLASS (taskview_generic_proxy_parent_class)->constructed != NULL)
	  G_OBJECT_CLASS (taskview_generic_proxy_parent_class)->constructed (gobject);
}

static void
taskview_generic_proxy_dispose (GObject *gobject)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (gobject);

  if (self->priv->proxy != NULL)
    {
      g_object_unref (self->priv->proxy);
      self->priv->proxy = NULL;
    }

  if (self->priv->connection != NULL)
    {
      g_object_unref (self->priv->connection);
      self->priv->connection = NULL;
    }

  G_OBJECT_CLASS (taskview_generic_proxy_parent_class)->dispose (gobject);
}

static void
taskview_generic_proxy_finalize (GObject *gobject)
{
  TaskviewGenericProxy *self = TASKVIEW_GENERIC_PROXY (gobject);

  g_free (self->priv->dbus_name);
  g_free (self->priv->object_path);

  G_OBJECT_CLASS (taskview_generic_proxy_parent_class)->finalize (gobject);
}

static void
taskview_generic_proxy_class_init (TaskviewGenericProxyClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  g_type_class_add_private (gobject_class, sizeof (TaskviewGenericProxyPrivate));

  gobject_class->constructed = taskview_generic_proxy_constructed;
  gobject_class->dispose = taskview_generic_proxy_dispose;
  gobject_class->finalize = taskview_generic_proxy_finalize;
  gobject_class->set_property = taskview_generic_proxy_set_property;
  gobject_class->get_property = taskview_generic_proxy_get_property;

  GParamSpec *pspec;

  /* D-Bus Name */
  pspec = g_param_spec_string ("dbus-name",
                               "D-Bus Name",
                               "The D-Bus name of the remote application to monitor",
                               NULL,
                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DBUS_NAME, pspec);

  /* D-Bus Object Path */
  pspec = g_param_spec_string ("dbus-object-path",
                               "D-Bus Object Path",
                               "The D-Bus Object Path of the TaskView",
                               NULL,
                               G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DBUS_OBJECT_PATH, pspec);

  /* D-Bus Connection */
  pspec = g_param_spec_pointer ("dbus-connection",
                                "D-Bus connection",
                                "The D-Bus connection",
                                G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DBUS_CONNECTION, pspec);

  /* Properties of the remote object */
	
  /* Icon */
  pspec = g_param_spec_string ("icon",
                               "Icon",
                               "The icon of the TaskView",
                               "" /* default value */,
                               G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_ICON, pspec);

  /* Description */
  pspec = g_param_spec_string ("description",
                               "Description",
                               "The description of the TaskView",
                               "" /* default value */,
                               G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DESCRIPTION, pspec);

  /* Details */
  pspec = g_param_spec_string ("details",
                               "Details",
                               "The details of the TaskView",
                               "" /* default value */,
                               G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DETAILS, pspec);

  /* State */
  pspec = g_param_spec_int ("state",
                            "State",
                            "The state of the TaskView",
                            TASKVIEW_PREPARING,
                            TASKVIEW_CANCELLED,
                            TASKVIEW_RUNNING,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_STATE, pspec);

  /* Progress */
  pspec = g_param_spec_int ("progress",
                            "Progress",
                            "The progress of the TaskView",
                            -1  /* minimum value */,
                            100 /* maximum value */,
                            -1  /* default value */,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROGRESS, pspec);

  /* RemainingTime */
  pspec = g_param_spec_int ("remaining-time",
                            "Remaining Time",
                            "The estimated remaining time until completion of the TaskView",
                            -1,
                            G_MAXINT32,
                            -1,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_REMAINING_TIME, pspec);

  /* CreationTime */
  pspec = g_param_spec_int64 ("creation-time",
                            "Creation Time",
                            "The creation time of the TaskView",
                            -1,
                            G_MAXINT64,
                            -1,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CREATION_TIME, pspec);

  /* FinishedTime */
  pspec = g_param_spec_int64 ("finished-time",
                              "Finished Time",
                              "The finished time of the TaskView",
                              -1,
                              G_MAXINT64,
                              -1,
                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_FINISHED_TIME, pspec);

  /* Pausable  */
  pspec = g_param_spec_boolean ("viewable",
                                "Viewable",
                                "Does the worker provide a seperate dialogue?",
                                FALSE,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_VIEWABLE, pspec);
	
  /* Pausable  */
  pspec = g_param_spec_boolean ("pausable",
                                "Pausable",
                                "Is the task pausable?",
                                FALSE,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PAUSABLE, pspec);

  /* Cancellable */
  pspec = g_param_spec_boolean ("cancellable",
                                "Cancellable",
                                "Is the task cancellable?",
                                FALSE,
                                G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CANCELLABLE, pspec);
}

static void
taskview_generic_proxy_init (TaskviewGenericProxy *self)
{
  self->priv = TASKVIEW_GENERIC_PROXY_GET_PRIVATE (self);
  self->priv->proxy = NULL;
}

/* ----------------------------------------------------------------------------------------------------
 * Wrapper for remote methods
 * ---------------------------------------------------------------------------------------------------- */

static inline void
call (GDBusProxy *proxy, const gchar *method)
{
  g_return_if_fail (G_IS_DBUS_PROXY (proxy));
  g_dbus_proxy_call (proxy,
                     method,
                     g_variant_new ("()"),
                     G_DBUS_CALL_FLAGS_NONE,
                     -1,
                     NULL,
                     NULL,
                     NULL);
}

void
taskview_generic_proxy_resume (TaskviewGenericProxy *self)
{
  call (self->priv->proxy, "Resume");
}

void
taskview_generic_proxy_pause (TaskviewGenericProxy *self)
{
  call (self->priv->proxy, "Pause");
}

void
taskview_generic_proxy_cancel (TaskviewGenericProxy *self)
{
  call (self->priv->proxy, "Cancel");
}

void
taskview_generic_proxy_view (TaskviewGenericProxy *self)
{
  call (self->priv->proxy, "View");
}

/* ----------------------------------------------------------------------------------------------------
 * C Wrapper for g_object_get
 * ---------------------------------------------------------------------------------------------------- */

gchar *
taskview_generic_proxy_get_icon (TaskviewGenericProxy *self)
{
  gchar *icon;
  g_object_get (self, "icon", &icon, NULL);
  return icon;
}

gchar *
taskview_generic_proxy_get_description (TaskviewGenericProxy *self)
{
  gchar *description;
  g_object_get (self, "description", &description, NULL);
  return description;
}

gchar *
taskview_generic_proxy_get_details (TaskviewGenericProxy *self)
{
  gchar *details;
  g_object_get (self, "details", &details, NULL);
  return details;
}

gint
taskview_generic_proxy_get_state (TaskviewGenericProxy *self)
{
  gint state;
  g_object_get (self, "state", &state, NULL);
  return state;
}

gint
taskview_generic_proxy_get_progress (TaskviewGenericProxy *self)
{
  gint progress;
  g_object_get (self, "progress", &progress, NULL);
  return progress;
}

gint
taskview_generic_proxy_get_remaining_time (TaskviewGenericProxy *self)
{
  gint remaining_time;
  g_object_get (self, "remaining-time", &remaining_time, NULL);
  return remaining_time;
}

gint64
taskview_generic_proxy_get_creation_time (TaskviewGenericProxy *self)
{
  gint64 creation_time;
  g_object_get (self, "creation-time", &creation_time, NULL);
  return creation_time;
}

gint64
taskview_generic_proxy_get_finished_time (TaskviewGenericProxy *self)
{
  gint64 finished_time;
  g_object_get (self, "finished-time", &finished_time, NULL);
  return finished_time;
}
