/*
 * taskview-generic.c
 *
 * TaskviewGeneric
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING.LESSER file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include "taskview-generic.h"
#include "taskview-manager.h"
#include "taskview.h"

#define TASKVIEW_GENERIC_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), TASKVIEW_TYPE_GENERIC, TaskviewGenericPrivate))

G_DEFINE_TYPE (TaskviewGeneric, taskview_generic, G_TYPE_OBJECT);

static GHashTable *instances = NULL;
static guint instances_num = 0;

struct _TaskviewGenericPrivate
{
  TaskviewManager *manager;
  guint id;
  gchar *object_path;

  gchar *icon;
  gchar *description;
  gchar *details;
  gint state;
  gdouble progress;
  gint remaining_time;
  gint64 creation_time;
  gint64 finished_time;

  TaskviewCallback view;
  TaskviewCallback resume;
  TaskviewCallback pause;
  TaskviewCallback cancel;
  gpointer user_data;
};

/* ----------------------------------------------------------------------------------------------------
 * GDBus Interface Specification
 * ---------------------------------------------------------------------------------------------------- */

static GDBusNodeInfo *introspection_data = NULL;
static const gchar introspection_xml[] =
  "<node>"
  "  <interface name=\"org.gnome.TaskView.Generic\">"
  "    <property type=\"s\" name=\"Icon\"              access=\"read\" />"
  "    <property type=\"s\" name=\"Description\"       access=\"read\" />"
  "    <property type=\"s\" name=\"Details\"           access=\"read\" />"
  "    <property type=\"i\" name=\"State\"             access=\"read\" />"
  "    <property type=\"i\" name=\"Progress\"          access=\"read\" />"
  "    <property type=\"i\" name=\"RemainingTime\"     access=\"read\" />"
  "    <property type=\"x\" name=\"CreationTime\"      access=\"read\" />"
  "    <property type=\"x\" name=\"FinishedTime\"      access=\"read\" />"
  "    <method name=\"View\" />"
  "    <method name=\"Resume\" />"
  "    <method name=\"Pause\" />"
  "    <method name=\"Cancel\" />"
  "    <property type=\"b\" name=\"Viewable\" access=\"read\" />"
  "    <property type=\"b\" name=\"Pausable\" access=\"read\" />"
  "    <property type=\"b\" name=\"Cancellable\" access=\"read\" />"
  "  </interface>"
  "</node>";

/* ----------------------------------------------------------------------------------------------------
 * GObject Properties
 * ---------------------------------------------------------------------------------------------------- */

enum
{
  PROP_0,

  PROP_ICON,
  PROP_DESCRIPTION,
  PROP_DETAILS,
  PROP_STATE,
  PROP_PROGRESS,
  PROP_REMAINING_TIME,
  PROP_CREATION_TIME,
  PROP_FINISHED_TIME,
    
  PROP_CALLBACK_PAUSE,
  PROP_CALLBACK_CANCEL,
  PROP_CALLBACK_VIEW,
  PROP_CALLBACK_RESUME,
  PROP_USER_DATA,

  PROP_OBJECT_PATH,
  PROP_ID
};

static void
taskview_generic_set_property (GObject      *object,
                               guint         property_id,
                               const GValue *value,
                               GParamSpec   *pspec)
{
  TaskviewGeneric *self = TASKVIEW_GENERIC (object);
  GVariantBuilder *invalidated_properties;
  GVariantBuilder *changed_properties;
  GError *error = NULL;

  invalidated_properties = g_variant_builder_new (G_VARIANT_TYPE("as"));
  changed_properties = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);

  switch (property_id)
    {
    case PROP_ICON:
      if (g_strcmp0 (self->priv->icon, g_value_get_string (value)) == 0) return;
      g_free (self->priv->icon);
      self->priv->icon = g_value_dup_string (value);
      g_variant_builder_add (changed_properties, "{sv}", "Icon", g_variant_new_string (self->priv->icon));
      break;

    case PROP_DESCRIPTION:
      if (g_strcmp0 (self->priv->description, g_value_get_string (value)) == 0) return;
      g_free (self->priv->description);
      self->priv->description = g_value_dup_string (value);
      g_variant_builder_add (changed_properties, "{sv}", "Description", g_variant_new_string (self->priv->description));
      break;

    case PROP_DETAILS:
      if (g_strcmp0 (self->priv->details, g_value_get_string (value)) == 0) return;
      g_free (self->priv->details);
      self->priv->details = g_value_dup_string (value);
      g_variant_builder_add (changed_properties, "{sv}", "Details", g_variant_new_string (self->priv->details));
      break;

    case PROP_STATE:
      if (self->priv->state == g_value_get_int (value)) return;
      self->priv->state = g_value_get_int (value);
      g_variant_builder_add (changed_properties, "{sv}", "State", g_variant_new_int32 (self->priv->state));
      if (self->priv->state == TASKVIEW_FINISHED && self->priv->finished_time == -1)
        {
          GTimeVal time;
          g_get_current_time (&time);
          self->priv->finished_time = time.tv_sec;
          g_variant_builder_add (changed_properties, "{sv}", "FinishedTime", g_variant_new_int64 (self->priv->finished_time));
        }
      break;

    case PROP_PROGRESS:
      if (self->priv->progress == g_value_get_int (value)) return;
      self->priv->progress = g_value_get_int (value);
      g_variant_builder_add (changed_properties, "{sv}", "Progress", g_variant_new_int32 (self->priv->progress));
      break;

    case PROP_REMAINING_TIME:
      if (self->priv->remaining_time == g_value_get_int (value)) return;
      self->priv->remaining_time = g_value_get_int (value);
      g_variant_builder_add (changed_properties, "{sv}", "RemainingTime", g_variant_new_int32 (self->priv->remaining_time));
      break;

    case PROP_CALLBACK_VIEW:
      if (self->priv->view == g_value_get_pointer (value)) return;
      self->priv->view = g_value_get_pointer (value);
      g_variant_builder_add (changed_properties, "{sv}", "Viewable", g_variant_new_boolean (self->priv->view != NULL));
      break;

    case PROP_CALLBACK_PAUSE:
      if (self->priv->pause == g_value_get_pointer (value)) return;
      self->priv->pause = g_value_get_pointer (value);
      g_variant_builder_add (changed_properties, "{sv}", "Pausable", g_variant_new_boolean (self->priv->pause != NULL));
      break;

    case PROP_CALLBACK_CANCEL:
      if (self->priv->cancel == g_value_get_pointer (value)) return;
      self->priv->cancel = g_value_get_pointer (value);
      g_variant_builder_add (changed_properties, "{sv}", "Cancellable", g_variant_new_boolean (self->priv->cancel != NULL));
      break;

    case PROP_CALLBACK_RESUME:
      self->priv->resume = g_value_get_pointer (value);
      break;

    case PROP_USER_DATA:
      self->priv->user_data = g_value_get_pointer (value);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }

  /* FIXME: Bad code */
  if (property_id <= PROP_CALLBACK_CANCEL)
    taskview_manager_property_changed (self->priv->manager, self->priv->object_path, TASKVIEW_GENERIC_INTERFACE_NAME, changed_properties, invalidated_properties);

  g_variant_builder_unref (changed_properties);
  g_variant_builder_unref (invalidated_properties);
}

static void
taskview_generic_get_property (GObject    *object,
                               guint       property_id,
                               GValue     *value,
                               GParamSpec *pspec)
{
  TaskviewGeneric *self = TASKVIEW_GENERIC (object);

  switch (property_id)
    {
    case PROP_ICON:
      g_value_set_string (value, self->priv->icon);
      break;

    case PROP_DESCRIPTION:
      g_value_set_string (value, self->priv->description);
      break;

    case PROP_DETAILS:
      g_value_set_string (value, self->priv->details);
      break;

    case PROP_STATE:
      g_value_set_int (value, self->priv->state);
      break;

    case PROP_PROGRESS:
      g_value_set_int (value, self->priv->progress);
      break;

    case PROP_REMAINING_TIME:
      g_value_set_int (value, self->priv->remaining_time);
      break;

    case PROP_CREATION_TIME:
      g_value_set_int64 (value, self->priv->creation_time);
      break;

    case PROP_FINISHED_TIME:
      g_value_set_int64 (value, self->priv->finished_time);
      break;

    case PROP_OBJECT_PATH:
      g_value_set_string (value, self->priv->object_path);
      break;

    case PROP_ID:
      g_value_set_uint (value, self->priv->id);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }
}

/* ----------------------------------------------------------------------------------------------------
 * D-Bus Methods
 * ---------------------------------------------------------------------------------------------------- */

static void
handle_method_call (GDBusConnection       *connection,
                    const gchar           *sender,
                    const gchar           *object_path,
                    const gchar           *interface_name,
                    const gchar           *method_name,
                    GVariant              *parameters,
                    GDBusMethodInvocation *invocation,
                    gpointer               user_data)
{
  /* FIXME: Handle error returned from the callbacks... */
  TaskviewGeneric *self;
  GError *error;

  g_assert (TASKVIEW_IS_GENERIC (user_data));
  self = TASKVIEW_GENERIC (user_data);

  if (g_strcmp0 (method_name, "View") == 0 && self->priv->view)
    {
      self->priv->view (self, &error, self->priv->user_data);
      g_dbus_method_invocation_return_value (invocation, NULL);
    }
  else if (g_strcmp0 (method_name, "Resume") == 0 && self->priv->resume)
    {
      g_dbus_method_invocation_return_value (invocation, NULL);
      self->priv->resume (self, &error, self->priv->user_data);
    }
  else if (g_strcmp0 (method_name, "Pause") == 0 && self->priv->pause)
    {
      g_dbus_method_invocation_return_value (invocation, NULL);
      self->priv->pause (self, &error, self->priv->user_data);
    }
  else if (g_strcmp0 (method_name, "Cancel") == 0 && self->priv->cancel)
    {
      self->priv->cancel (self, &error, self->priv->user_data);
      g_dbus_method_invocation_return_value (invocation, NULL);
    }
  else
    {
      g_dbus_method_invocation_return_dbus_error (invocation,
          "org.gnome.TaskView.CallbackMissing",
          "No Callback is set");
    }
}

static GVariant *
handle_get_property (GDBusConnection  *connection,
                     const gchar      *sender,
                     const gchar      *object_path,
                     const gchar      *interface_name,
                     const gchar      *property_name,
                     GError          **error,
                     gpointer          user_data)
{
  TaskviewGeneric *self;
  GVariant *ret;

  g_assert (TASKVIEW_IS_GENERIC (user_data));
  self = TASKVIEW_GENERIC (user_data);

  ret = NULL;

  if (g_strcmp0 (property_name, "Icon") == 0)
    {
      ret = g_variant_new_string (self->priv->icon);
    }
  else if (g_strcmp0 (property_name, "Description") == 0)
    {
      ret = g_variant_new_string (self->priv->description);
    }
  else if (g_strcmp0 (property_name, "Details") == 0)
    {
      ret = g_variant_new_string (self->priv->details);
    }
  else if (g_strcmp0 (property_name, "State") == 0)
    {
      ret = g_variant_new_int32 (self->priv->state);
    }
  else if (g_strcmp0 (property_name, "Progress") == 0)
    {
      ret = g_variant_new_int32 (self->priv->progress);
    }
  else if (g_strcmp0 (property_name, "RemainingTime") == 0)
    {
      ret = g_variant_new_int32 (self->priv->remaining_time);
    }
  else if (g_strcmp0 (property_name, "CreationTime") == 0)
    {
      ret = g_variant_new_int64 (self->priv->creation_time);
    }
  else if (g_strcmp0 (property_name, "FinishedTime") == 0)
    {
      ret = g_variant_new_int64 (self->priv->finished_time);
    }  
  else if (g_strcmp0 (property_name, "Viewable") == 0)
    {
      ret = g_variant_new_boolean (self->priv->view != NULL);
    }
  else if (g_strcmp0 (property_name, "Pausable") == 0)
    {
      ret = g_variant_new_boolean (self->priv->pause != NULL);
    }
  else if (g_strcmp0 (property_name, "Cancellable") == 0)
    {
      ret = g_variant_new_boolean (self->priv->cancel != NULL);
    }
  else
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "Unknown Property");
    }

  return ret;
}

const GDBusInterfaceVTable taskview_generic_interface_vtable =
{
  handle_method_call,
  handle_get_property,
  NULL
};

/* ----------------------------------------------------------------------------------------------------
 * GObject
 * ---------------------------------------------------------------------------------------------------- */

static void
taskview_generic_dispose (GObject *gobject)
{
  TaskviewGeneric *self = TASKVIEW_GENERIC (gobject);

  g_hash_table_remove (instances, &self->priv->id);

  if (self->priv->manager != NULL)
    {
      taskview_manager_tv_changed (self->priv->manager, self->priv->object_path, FALSE);
      g_object_unref (self->priv->manager);
      self->priv->manager = NULL;
    }

  G_OBJECT_CLASS (taskview_generic_parent_class)->dispose (gobject);
}

static void
taskview_generic_finalize (GObject *gobject)
{
  TaskviewGeneric *self = TASKVIEW_GENERIC (gobject);

  g_free (self->priv->icon);
  g_free (self->priv->description);
  g_free (self->priv->object_path);

  G_OBJECT_CLASS (taskview_generic_parent_class)->finalize (gobject);
}

static void
taskview_generic_class_init (TaskviewGenericClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
  GParamSpec *pspec;

  /* Build D-Bus Introspection */
  introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
  g_assert (introspection_data != NULL);
  klass->interface = g_dbus_node_info_lookup_interface (introspection_data, TASKVIEW_GENERIC_INTERFACE_NAME);
  g_assert (klass->interface);
  klass->vtable = &taskview_generic_interface_vtable;

  instances = g_hash_table_new (g_int_hash, g_int_equal);

  g_type_class_add_private (gobject_class, sizeof (TaskviewGenericPrivate));

  gobject_class->dispose = taskview_generic_dispose;
  gobject_class->finalize = taskview_generic_finalize;
  gobject_class->set_property = taskview_generic_set_property;
  gobject_class->get_property = taskview_generic_get_property;

  /* Icon */
  pspec = g_param_spec_string ("icon",
                               "Icon",
                               "The icon of the TaskView",
                               "" /* default value */,
                               G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_ICON, pspec);

  /* Description */
  pspec = g_param_spec_string ("description",
                               "Description",
                               "The description of the TaskView",
                               "" /* default value */,
                               G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DESCRIPTION, pspec);

  /* Details */
  pspec = g_param_spec_string ("details",
                               "Details",
                               "The details of the TaskView",
                               "" /* default value */,
                               G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DETAILS, pspec);

  /* State */
  pspec = g_param_spec_int ("state",
                            "State",
                            "The state of the TaskView",
                            TASKVIEW_PREPARING,
                            TASKVIEW_CANCELLED,
                            TASKVIEW_RUNNING,
                            G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_STATE, pspec);

  /* Progress */
  pspec = g_param_spec_int ("progress",
                            "Progress",
                            "The progress of the TaskView",
                            -1  /* minimum value */,
                            100 /* maximum value */,
                            -1  /* default value */,
                            G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROGRESS, pspec);

  /* RemainingTime */
  pspec = g_param_spec_int ("remaining-time",
                            "Remaining Time",
                            "The estimated remaining time until completion of the TaskView",
                            -1,
                            G_MAXINT32,
                            -1,
                            G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_REMAINING_TIME, pspec);

  /* CreationTime */
  pspec = g_param_spec_int64 ("creation-time",
                            "Creation Time",
                            "The creation time of the TaskView",
                            -1,
                            G_MAXINT64,
                            -1,
                            G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CREATION_TIME, pspec);

  /* FinishedTime */
  pspec = g_param_spec_int64 ("finished-time",
                              "Finished Time",
                              "The finished time of the TaskView",
                              -1,
                              G_MAXINT64,
                              -1,
                              G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_FINISHED_TIME, pspec);

  /* View Callback */
  pspec = g_param_spec_pointer ("callback-view",
                                "callback-view",
                                "Function called when view is requested",
                                G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CALLBACK_VIEW, pspec);

  /* Resume Callback */
  pspec = g_param_spec_pointer ("callback-resume",
                                "callback-resume",
                                "Function called when resuming the task is requested",
                                G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CALLBACK_RESUME, pspec);

  /* Pause Callback */
  pspec = g_param_spec_pointer ("callback-pause",
                                "callback-pause",
                                "Function called when pausing the task is requested",
                                G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CALLBACK_PAUSE, pspec);

  /* Cancel Callback */
  pspec = g_param_spec_pointer ("callback-cancel",
                                "callback-cancel",
                                "Function called when pausing the task is requested",
                                G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CALLBACK_CANCEL, pspec);

  /* User data */
  pspec = g_param_spec_pointer ("user-data",
                                "user data",
                                "Data passed to the callbacks",
                                G_PARAM_CONSTRUCT | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_USER_DATA, pspec);

  /* Object path */
  pspec = g_param_spec_string ("object-path",
                               "object path",
                               "The object path of the TaskView",
                               "",
                               G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_OBJECT_PATH, pspec);

  /* ID */
  pspec = g_param_spec_uint ("id",
                             "ID",
                             "The id of the TaskView",
                             0,
                             G_MAXUINT,
                             0,
                             G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_ID, pspec);
}

static void
taskview_generic_init (TaskviewGeneric *self)
{
  TaskviewGenericClass *klass = TASKVIEW_GENERIC_GET_CLASS (self);
  GTimeVal time;
  
  gchar *str_id;

  self->priv = TASKVIEW_GENERIC_GET_PRIVATE (self);

  instances_num++;
  self->priv->id = instances_num;

  str_id = g_strdup_printf ("%i", self->priv->id);
  self->priv->object_path = g_strconcat (TASKVIEW_SUBTREE_OBJECTPATH, "/", str_id, NULL);
  g_free (str_id);

  self->priv->icon = g_strdup ("");
  self->priv->description = g_strdup ("");
  self->priv->details = g_strdup ("");
  self->priv->manager = taskview_manager_dup ();

  g_get_current_time (&time);
  self->priv->creation_time = time.tv_sec;
  self->priv->finished_time = -1;
  
  g_hash_table_insert (instances, &self->priv->id, self);
  taskview_manager_tv_changed (self->priv->manager, self->priv->object_path, TRUE);
}

/* ----------------------------------------------------------------------------------------------------
 * Accessor functions
 * ---------------------------------------------------------------------------------------------------- */

GHashTable *
taskview_generic_get_views (void)
{
  return instances;
}

TaskviewGeneric *
taskview_generic_get_inst_by_id (guint id)
{
  gpointer swap;

  swap = g_hash_table_lookup (instances, &id);
  if (TASKVIEW_IS_GENERIC (swap))
    return TASKVIEW_GENERIC (swap);
  else
    return NULL;
}

/* ----------------------------------------------------------------------------------------------------
 * C Wrapper for g_object_set/get/new
 * ---------------------------------------------------------------------------------------------------- */

TaskviewGeneric *
taskview_generic_new (void)
{
  return g_object_new (TASKVIEW_TYPE_GENERIC, NULL);
}

gchar *
taskview_generic_get_icon (TaskviewGeneric *self)
{
  gchar *icon;
  g_object_get (self, "icon", &icon, NULL);
  return icon;
}

gchar *
taskview_generic_get_description (TaskviewGeneric *self)
{
  gchar *description;
  g_object_get (self, "description", &description, NULL);
  return description;
}

gchar *
taskview_generic_get_details (TaskviewGeneric *self)
{
  gchar *details;
  g_object_get (self, "details", &details, NULL);
  return details;
}

gint
taskview_generic_get_state (TaskviewGeneric *self)
{
  gint state;
  g_object_get (self, "state", &state, NULL);
  return state;
}

gint
taskview_generic_get_progress (TaskviewGeneric *self)
{
  gint progress;
  g_object_get (self, "progress", &progress, NULL);
  return progress;
}

gint
taskview_generic_get_remaining_time (TaskviewGeneric *self)
{
  gint remaining_time;
  g_object_get (self, "remaining-time", &remaining_time, NULL);
  return remaining_time;
}

gint64
taskview_generic_get_creation_time (TaskviewGeneric *self)
{
  gint64 creation_time;
  g_object_get (self, "creation-time", &creation_time, NULL);
  return creation_time;
}

gint64
taskview_generic_get_finished_time (TaskviewGeneric *self)
{
  gint64 finished_time;
  g_object_get (self, "finished-time", &finished_time, NULL);
  return finished_time;
}

gchar *
taskview_generic_get_object_path (TaskviewGeneric *self)
{
  gchar *object_path;
  g_object_get (self, "object-path", &object_path, NULL);
  return object_path;
}

guint
taskview_generic_get_id (TaskviewGeneric *self)
{
  guint id;
  g_object_get (self, "id", &id, NULL);
  return id;
}

void
taskview_generic_set_icon (TaskviewGeneric *self, const gchar *icon)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "icon", icon, NULL);
}

void
taskview_generic_set_description (TaskviewGeneric *self, const gchar *description)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "description", description, NULL);
}

void
taskview_generic_set_details (TaskviewGeneric *self, const gchar *details)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "details", details, NULL);
}

void
taskview_generic_set_state (TaskviewGeneric *self, gint state)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "state", state, NULL);
}

void
taskview_generic_set_progress (TaskviewGeneric *self, gint progress)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "progress", progress, NULL);
}

void
taskview_generic_set_remaining_time (TaskviewGeneric *self, gint remaining_time)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "remaining-time", remaining_time, NULL);
}

void
taskview_generic_set_view_callback (TaskviewGeneric *self, TaskviewCallback *view)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "callback-view", &view, NULL);
}

void
taskview_generic_set_resume_callback (TaskviewGeneric *self, TaskviewCallback *resume)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "callback-resume", &resume, NULL);
}

void
taskview_generic_set_pause_callback (TaskviewGeneric *self, TaskviewCallback *pause)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "callback-pause", &pause, NULL);
}

void
taskview_generic_set_cancel_callback (TaskviewGeneric *self, TaskviewCallback *cancel)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "callback-cancel", &cancel, NULL);
}

void
taskview_generic_set_user_data (TaskviewGeneric *self, gpointer user_data)
{
  g_return_if_fail (TASKVIEW_IS_GENERIC (self));
  g_object_set (self, "user-data", &user_data, NULL);
}
