/*
 * sample_worker.c
 *
 * Example to illustrate the use of the worker components.
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include <glib.h>
#include "../libtaskview/taskview.h"
//#include <libtaskview/taskview.h>

/*
 * This function is called if the Resume method of the D-Bus API is invoked.
 */
void
resume_callback (TaskviewGeneric *origin, GError **error, gpointer user_data)
{
  // Handle the request
  g_print ("Resume was invoked!");

  // Set the the state to running
  g_object_set (origin, "state", TASKVIEW_RUNNING, NULL);
}

/*
 * This function is called if the Pause method of the D-Bus API is invoked.
 */
void
pause_callback (TaskviewGeneric *origin, GError **error, gpointer user_data)
{
  // Handle the request
  g_print ("Pause was invoked!");

  // Set the the state to paused
  g_object_set (origin, "state", TASKVIEW_PAUSED, NULL);
}

/*
 * This function is called if the Cancel method of the D-Bus API is invoked.
 */
void
cancel_callback (TaskviewGeneric *origin, GError **error, gpointer user_data)
{
  // Handle the request, e.g destroy I/O objects
  g_print ("Cancel was invoked!");

  // Set the the state to canceled
  g_object_set (origin, "state", TASKVIEW_CANCELLED, NULL);

  // Optionally we can dispose our TaskviewGeneric object here (or later...)
  g_object_unref (G_OBJECT (origin));
}

/*
 * This function is called if the View method of the D-Bus API is invoked.
 */
void
view_callback (TaskviewGeneric *origin, GError **error, gpointer user_data)
{
  // Handle the request, e.g show a dialog.
  g_print ("View was invoked!");  
}


/* These procedures emulate the progress of an Task */ 
gboolean
run_generic (gpointer user_data)
{
  g_return_val_if_fail (TASKVIEW_IS_GENERIC (user_data), FALSE);
  TaskviewGeneric *tv = TASKVIEW_GENERIC (user_data);

  /* Emulation */
  gint progress, state, time;

  state = taskview_generic_get_state (tv);
  progress = taskview_generic_get_progress (tv);
  time = taskview_generic_get_remaining_time (tv);

  if (state != TASKVIEW_RUNNING)
    return TRUE;

  progress++;
  time--;

  taskview_generic_set_progress (tv, progress);
  taskview_generic_set_remaining_time (tv, time);

  if (progress == 100)
    taskview_generic_set_state (tv, TASKVIEW_FINISHED);

  return TRUE;
}

gboolean
run_io (gpointer user_data)
{
  g_return_val_if_fail (TASKVIEW_IS_IO (user_data), FALSE);
  TaskviewIO *tv = TASKVIEW_IO (user_data);

  gint state;
  guint64 total, processed;

  state = taskview_generic_get_state (TASKVIEW_GENERIC (tv));
  processed = taskview_io_get_processed_size (tv);
  total = taskview_io_get_total_size (tv);

  if (state != TASKVIEW_RUNNING)
    return TRUE;

  processed += 10;

  taskview_io_set_processed_size (tv, processed);

  if (processed == total)
  {
    taskview_io_set_processed_items (tv, 1); 
    taskview_generic_set_state (TASKVIEW_GENERIC (tv), TASKVIEW_FINISHED);
  }

  return TRUE;
}

int
main (int argc, char *argv[])
{
  GMainLoop *loop;
  TaskviewGeneric *task_gen;
  TaskviewIO *task_io;

  g_type_init ();
  loop = g_main_loop_new (NULL, FALSE);

  // The library uses these strings for D-Bus and 
  // exports the Application Name to the observer.   
  g_set_application_name ("Sample Worker");
  g_set_prgname ("sample_worker");

  // Create a new object and set all properties	
  task_gen = g_object_new (TASKVIEW_TYPE_GENERIC,
                           "icon", "printer",
                           "description", "Printing \"foobar.pdf\" on \"L. Laser Corp. Printer\"",
                           "details", "3 of 15 pages",
                           "state", TASKVIEW_RUNNING,
                           "progress", 0,
                           "remaining-time", 100LL,
                           NULL);

  gchar *source_uri[] = { "/home/bob/123.c", NULL };
  gchar *dest_uri[] = { "/home/alice/123.c", NULL };

  task_io = g_object_new (TASKVIEW_TYPE_IO,
                          "icon", "file-manager",
                          "description", "Moving \"123.c\" to \"/home/alice\"",
                          "state", TASKVIEW_RUNNING,
                          // Setting callbacks, this must be done during initialization.
                          "callback-resume", resume_callback,
                          "callback-pause", pause_callback,
                          "callback-cancel", cancel_callback,
                          "callback-view", view_callback,
                          // IO specific values
                          "type", TASKVIEW_IO_MOVE,
                          "current-item", 0,
                          "destination-uri", dest_uri,
                          "source-uri", source_uri,
                          // The library uses these values to calculate the progress 
                          // and the remaining time.
                          // To disable this behavior, set: "auto-update", FALSE 
                          "processed-items", 0,
                          "total-items", 1,
                          "processed-size", 0,
                          "total-size", 1000,
                          NULL);

  // Register emulation procedures at the mainloop, not needed in your code.
  g_timeout_add (1000, run_generic, task_gen);
  g_timeout_add (500, run_io, task_io);

  // If you're not in a mainloop, you have to start it, otherwise GDBus is not available!
  g_main_loop_run (loop);

  return 0;
}
